# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2010-2012 ASPerience SARL (<http://www.asperience.fr>). 
#    All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from mx import DateTime
import time
import netsvc
from osv import fields, osv
from tools import config
from tools.translate import _
import tools
import datetime
import os
import csv
import re
from tools import ustr
import codecs, cStringIO
import shutil
from ftplib import FTP
import base64
import threading, thread
import pooler
import traceback
import sys
import logging
_logger = logging.getLogger(__name__)

def dict2xml(root,level=0):
	level_xml = '\t'*level
	xml = ""
	for key in root.keys():
		if isinstance(root[key], dict):
			xml = '%s%s<%s>\n%s%s</%s>\n' % (xml, level_xml, key, dict2xml(root[key],level+1), level_xml, key)
		elif isinstance(root[key], list):
			xml = '%s%s<%s>\n' % (xml, level_xml, key)
			for item in root[key]:
				xml = '%s%s' % (xml, dict2xml(item,level+1))
			xml = '%s%s</%s>\n' % (xml, level_xml, key)
		else:
			value = root[key]
			xml = '%s%s<%s>%s</%s>\n' % (xml, level_xml, key, value, key)
	return xml

def unicode_csv_reader(unicode_csv_data, dialect=csv.excel, **kwargs):
	csv_reader = csv.reader(utf_8_encoder(unicode_csv_data), dialect=dialect, **kwargs)
	for row in csv_reader:
		yield [unicode(cell, 'utf-8') for cell in row]

class UnicodeWriter:
	def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
		self.queue = cStringIO.StringIO()
		self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
		self.stream = f
		self.encoder = codecs.getincrementalencoder(encoding)()

	def writerow(self, row):
		row2 = []
		for s in row:
			if isinstance(s,unicode):
				s2 = s
				for i in s :
					if ord(i) > 255 :
						s2 = s2.replace(i,'')
				row2.append(s2.encode("utf-8"))
			else:
				row2.append(s)
		self.writer.writerow(row2)
		data = self.queue.getvalue()
		data = data.decode("utf-8")
		data = self.encoder.encode(data)
		self.stream.write(data)
		self.queue.truncate(0)

	def writerows(self, rows):
		for row in rows:
			self.writerow(row)
			
def utf_8_encoder(unicode_csv_data):
	for line in unicode_csv_data:
		yield line.encode('utf-8')
			
def stripnulls(data, parameters, length, type, cast):
	if type == 'float':
		data = data.replace(".",parameters['separator'])
	data = data.replace("\00", "").strip()
	while len(data) and data[-1] == ' ':
		data=data[:-2]
	return data

def fillleft(data, parameters, length, type, cast):
#	print "fillleft ",data, length, type, cast
	data = ustr(data)
	if type == 'float':
		data = data.replace(".",parameters['separator'])
	length+=1
	if len(data)>length : 
		data=data[:length]
	while len(data)<length:
		data=" "+data
	return data

def fillright(data, parameters, length, type, cast):
#	print "fillright ",data, length, type, cast
	data = ustr(data)
	if type == 'float':
		data = data.replace(".",parameters['separator'])
	length+=1
	if len(data)>length : 
		data=data[:length]
	while len(data)<length:
		data=data+" "
	return data

def fillzero(data, parameters, length, type, cast):
	length+=1
	if type == 'float':
		before = re.compile('\(\d+\,').findall(cast)
		after = re.compile('\,\d+\)').findall(cast)
		if before and len(before) == 1 and len(before[0]) > 2 and after and len(after) == 1 and len(after[0]) > 2:
			before = before[0][1:-1]
			after = after[0][1:-1]
			caststr = '%'+before+'.'+after+'f' 
			data = caststr % data
			data = data.replace('.',parameters['separator'])
			data = data.replace(' ',"0")
			
	if len(ustr(data))>length : 
		data=ustr(data)[:length]
	while len(ustr(data))<length:
		data="0"+str(data)
#	print data
	return data

	
class edi_edi (osv.osv):
	_name = 'edi.edi'
	_description = 'edi.edi'
	
	def _length(self, cr, uid, ids, name, arg, context=None):
		res = {}
		for edi in self.browse(cr, uid, ids, context=context):
			if edi.stop_identifier <= 0  and edi.start_identifier <= 0:
				res[edi.id] = 0
			else:
				res[edi.id] = edi.stop_identifier - edi.start_identifier + 1
		return res
	
	_columns = {
		'name':fields.char('Name', size=128, required=True),
		'code':fields.char('Code', size=128, required=True),
		'start_identifier':fields.integer('Start identifier', required=True),
		'stop_identifier':fields.integer('Stop identifier', required=True),
		'length_identifier': fields.function(_length, readonly=True, method=True, store=False, string='Length identifier', type='integer'),
		'line_edi_ids':fields.one2many('edi.edi.line.edi', 'edi', 'Line Edi'),
		'log_ids':fields.one2many('edi.edi.log', 'edi', 'Log'),
		'result_ids':fields.one2many('edi.edi.result', 'edi', 'Result'),
		'file_template':fields.char('Template File (Regular Expression)',128),
		'type': fields.selection([('edi','edi'),('csv','csv'),('xml','xml')], 'Type', required=True, size=24),
		
		'charset': fields.selection([('UTF-8','UTF-8'),('ISO-8859-1','ISO-8859-1')], 'Charset', required=True, size=24),
		'end_line': fields.selection([('simple','\\n'),('win','\\r\\n')], 'End line', required=True, size=24),
		
		'float_separator': fields.selection([(',',','),('.','.')], 'Float Separator', required=True, size=5),
		'eval_in': fields.text('IN'),
		'eval_out': fields.text('OUT'),
		'path_in':fields.char('Path In', size=128,),
		'path_in_move':fields.char('Path In Move', size=128,),
		'path_out':fields.char('Path Out', size=128,),
		'force':fields.boolean('Force'),
		
		'skip_first':fields.boolean('Skip the First'),
		'thread':fields.boolean('Progress',readonly=True),
		'background':fields.boolean('Background'),
		'delimiter':fields.char('Delimiter', size=1, required=True),
		'quotechar':fields.char('Quotechar', size=1, required=True),
		'fields_ids':fields.one2many('edi.edi.csv.field', 'edi', 'Fields'),
		'fields_xml_ids':fields.one2many('edi.edi.xml.field', 'edi', 'Fields'),
		'line_start':fields.integer('Start line', required=True),
		'line_stop':fields.integer('Stop line', required=True),
		
		'copy_in':fields.boolean('Copy before import'),
		'ftp_path_in':fields.char('Path In', size=128,),
		'ftp_url_in':fields.char('Url In', size=128),
		'ftp_login_in':fields.char('Login In', size=128),
		'ftp_password_in':fields.char('password', size=128),

		'copy_out':fields.boolean('Copy after export (not implemented)'),
		'ftp_path_out':fields.char('Path Out (not implemented)', size=128,),
		'ftp_url_out':fields.char('Url Out (not implemented)', size=128),
		'ftp_login_out':fields.char('Login Out (not implemented)', size=128),
		'ftp_password_out':fields.char('Password Out (not implemented)', size=128),
	}
	_defaults = {
		'path_in': lambda *a: './addons/asperience_edi/data_import/',
		'path_out': lambda *a: './addons/asperience_edi/data_export/',
		'start_identifier': lambda *a: 0,
		'stop_identifier': lambda *a: 0,
		'line_start': lambda *a: -1,
		'line_stop': lambda *a: -1,
		'delimiter': lambda *a: ";",
		'quotechar': lambda *a: "\"",
		'file_template': lambda *a: ".*",
		'float_separator': lambda *a: ".",
		'end_line': lambda *a: "simple",
		'charset': lambda *a: "UTF-8",
	}
	
	def _create_struct_edi(self, cr, uid, ids, context=False):
		result = {}
		for edi in self.browse(cr,uid,ids,context):
			result[edi.id] = {}
			for line in edi.line_edi_ids:
				result[edi.id][str(line.name)] = {}
				if edi.end_line == 'win':
					edi.end_line = '\r\n'
				else:
					edi.end_line = '\n'
					
				result[edi.id]["<<>>"] = {
						'start':edi.start_identifier,
						'stop':edi.stop_identifier,
						'separator':edi.float_separator,
						'charset':edi.charset,
						'end_line':edi.end_line,
				}
				for field in line.fields_ids:
					result[edi.id][str(line.name)][str(field.name)] = {
						'start':field.start,
						'stop':field.stop,
						'align': field.align,
						'type': field.type,
						'cast':field.cast,
					}
		return result
	
	def _create_struct_csv(self, cr, uid, ids, context=False):
		result = {}
		for edi in self.browse(cr,uid,ids,context):
			result[edi.id] = {}
			if edi.end_line == 'win':
				edi.end_line = '\r\n'
			else:
				edi.end_line = '\n'
			result[edi.id]["<<>>"] = {
					'separator':edi.float_separator,
					'charset':edi.charset,
					'end_line':edi.end_line,
			}
			for field in edi.fields_ids:
				result[edi.id][field.sequence-1] = {
						'name': str(field.name),
						'type': field.type,
						'cast': field.cast,
						'description': field.description,
				}
		return result
		
	def _create_attachment(self, cr, uid, ids, file_path, context=False):
		edi = self.browse(cr,uid,ids,context)[0]
		data_attach = {
			'name': os.path.basename(file_path),
			'datas':open(file_path, "rb").read().encode("base64"),
			'datas_fname': os.path.basename(file_path),
			'description': 'export',
			'res_model': 'edi.edi',
			'res_id': edi.id
		}
		self.pool.get('ir.attachment').create(cr,uid, data_attach)
			
			
	
	def _create_files(self, cr, uid, ids, data, context=False):
		edi = self.browse(cr,uid,ids,context)[0]
		for filename in data: 
			file = codecs.open(context['path_out']+filename,"w",edi.charset)
			for line in data[filename]:
				file.write(line)
			file.close()
			edi._create_attachment(file_path=context['path_out']+filename,context=context)
			if edi.copy_out:
				print "TODO copy out"
		return {}
	
	def _create_files_csv(self, cr, uid, ids, data, context=False):
		edi = self.browse(cr,uid,ids,context)[0]
		for filename in data: 
			if not data[filename]:
				continue
#			file = codecs.open(context['path_out']+filename,"w",edi.charset)
			file = open(context['path_out']+filename,"wb")
			csv_writer = UnicodeWriter(file, encoding=edi.charset, delimiter=str(edi.delimiter[0]), quotechar=str(edi.quotechar), quoting=csv.QUOTE_NONNUMERIC)
			for line in data[filename]:
				csv_writer.writerow(line)
			file.close()
			edi._create_attachment(file_path=context['path_out']+filename,context=context)
			if edi.copy_out:
				print "TODO copy out"
		return {}
	
	def _get_files(self, cr, uid, ids, path, template=False, context=False):
		edi = self.browse(cr,uid,ids,context)[0]
		_logger.info("_get_files : %s, patern : %s" % (path,template))
		result = []
		if edi.copy_in:
			print "TODO copy in"
		for i in os.listdir(path):
			if os.path.isfile(path+i) and i[0]!='.':
				if template and not re.compile(template).search(i):
					_logger.info("File : %s : Not Match" % (i)) 
					continue
				_logger.info("File : %s : Match" % (i))
				result.append(path+i)
		return result
		
	def _create_line(self, cr, uid, ids, line, type, structs, context=False):
		struct = structs[type]
		max = 0
		for field in struct:
			if struct[field]['stop'] > max :
				max = struct[field]['stop']
		outline = type+" "*(max-(len(type))) + structs['<<>>']['end_line']
		for field in struct:
			start = struct[field]['start']
			stop = struct[field]['stop']
			writeFunc = fillright
			if struct[field]['align'] == 'right':
				writeFunc = fillright
			elif struct[field]['align'] == 'left':
				writeFunc = fillleft
			elif struct[field]['align'] == 'zero':
				writeFunc = fillzero
			str_field = writeFunc(line[field], structs['<<>>'], stop-start, struct[field]['type'], struct[field]['cast'])
			outline = outline[0:start-1]+str_field+outline[stop:]
		return outline
	
	def _parse_line_edi(self, cr, uid, ids, line, structs, context=False):
		result = {}
		start_identifier = structs['<<>>']['start']-1
		stop_identifier = structs['<<>>']['stop']
		identifier = line[start_identifier:stop_identifier]
		
		if identifier not in structs:
			return False
		struct = structs[identifier]
		
		for key in struct :
			if key == '<<>>' :
				continue
			result[key] = False
		
		for field in struct:
			start = struct[field]['start']
			stop = struct[field]['stop']
			parseFunc = stripnulls
			result[field] = parseFunc(line[start-1:stop],structs['<<>>'], stop-start, struct[field]['type'], struct[field]['cast'])
			
		return identifier,result
	
	def _parse_line_csv(self, cr, uid, ids, line, structs, context=False):
		result = {}
		for key in structs:
			if len(line) > key:
				result[structs[key]['name']] = line[key].strip()
		return result

	def _search_warning(self, cr, uid, ids, model, filter, type='raise'):
		model_obj = self.pool.get(model)
		obj_ids = model_obj.search(cr,uid,filter)
		if len(obj_ids) != 1:
			if len(obj_ids) == 0:
				if type == 'log':
					_logger.info(_('%s Not Found with filter %s' % (model,filter)))
				elif type == 'raise':
					raise Exception(_('%s Not Found with filter %s' % (model,filter)))
				else:
					print '%s Not Found with filter %s' % (model,filter)
			else:
				if type == 'log':
					_logger.info(_('Many %s with filter %s' % (model,filter)))
				elif type == 'raise':
					raise Exception(_('Many %s with filter %s' % (model,filter)))
				else:
					print 'Many %s with filter %s' % (model,filter)
		else:
			obj_ids = obj_ids[0]
		return obj_ids
	
	def import_edi_thread(self, cr, uid, ids, context=False):
		cr2 = pooler.get_db(cr.dbname).cursor()
		thread_ptr = threading.Thread(target=self.import_edi, args=(cr2, uid, ids, context))
		thread_ptr.start()
		return {}
		
	def import_edi(self, cr, uid, ids, context=False):
		info = ""
		thread = False
		if "thread" in context:
			thread = context["thread"]
		cr2 = pooler.get_db(cr.dbname).cursor()
		cr2.autocommit(True)
		exception = False
		if not ids :
			if 'edi.code' in context:
				ids = self.search(cr,uid,[('code','=',context['edi.code'])])
			else:
				return False
		exception = False
		for edi in self.browse(cr,uid,ids,context):
			self.write(cr2,uid,[edi.id],{'thread':True})
			cr2.commit()
			try:
				structs = edi._create_struct_edi()[edi.id]
				if not edi.eval_in:
					continue
				data = {}
				for filename in edi._get_files(edi.path_in, edi.file_template):
					_logger.info("Import file : %s" % (filename))
					try:
						file = codecs.open(filename,'r',edi.charset)
						data[filename] = []
						for line in file:
							data[filename].append(edi._parse_line_edi(line,structs))
						edi.eval_in = edi.eval_in.replace(chr(13),"\n")
						context['edi'] = edi.id
						context['type'] = 'import'
						context['filename'] = filename
						exec edi.eval_in					
						file.close()
					except Exception, e:
						if not os.path.exists (edi.path_in+"error/"):
							os.makedirs(edi.path_in+"error/")
						shutil.move(edi.path_in+os.path.basename(filename), edi.path_in+"error/"+os.path.basename(filename))
						raise e
					if edi.path_in_move:
						shutil.move(edi.path_in+os.path.basename(filename), edi.path_in_move+os.path.basename(filename))
					_logger.info("End Import file : %s" % (filename))
			except Exception, e:
				import sys,traceback
				tb = sys.exc_info()
				tb_s = "".join(traceback.format_exception(*tb))
				info = tb_s
				try:
					info += "\n"+ustr(e.value)
				except:
					pass
#				info = reduce(lambda x, y: x+y, traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback))
				_logger.info("Except : %s" % (info))
				cr.rollback()
				self.pool.get('edi.edi.result').create(cr,uid,{"name":"_import_csv","value":info,"edi":edi.id})
				cr.commit()
				exception = True
		if not exception:
			cr.commit()
		for i in ids :
			cr2.execute("update edi_edi set thread = False where id = %s" % i)
		if thread :
			cr.close()
		cr2.close()
		if not thread and exception:
			raise Exception(info)
		return True
	
	def export_edi_thread(self, cr, uid, ids, context=False):
		cr2 = pooler.get_db(cr.dbname).cursor()
		thread_ptr = threading.Thread(target=self.export_edi, args=(cr2, uid, ids, context))
		thread_ptr.start()
		return {}
		
	def export_edi(self, cr, uid, ids, context=False):
		thread = False
		if "thread" in context:
			thread = context["thread"]
		cr2 = pooler.get_db(cr.dbname).cursor()
		cr2.autocommit(True)
		exception = False
		if not ids :
			if 'edi.code' in context:
				ids = self.search(cr,uid,[('code','=',context['edi.code'])])
			else:
				return False
			
		for edi in self.browse(cr,uid,ids,context):
			self.write(cr2,uid,[edi.id],{'thread':True})
			cr2.commit()
			try :
				structs = edi._create_struct_edi()[edi.id]
				if not edi.eval_out:
					continue
				edi.eval_out = edi.eval_out.replace(chr(13),"\n")
				data = {}
				context['edi'] = edi.id
				context['type'] = 'export'
				exec edi.eval_out 
				context['path_out'] = edi.path_out
				edi._create_files(data,context)
			except Exception, e:
				import sys,traceback
				tb = sys.exc_info()
				tb_s = "".join(traceback.format_exception(*tb))
				info = tb_s
#				info = ustr(e) +"\n" +reduce(lambda x, y: x+y, traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback))
				_logger.info("Except : %s" % (info))
				cr.rollback()
				self.pool.get('edi.edi.result').create(cr,uid,{"name":"_import_csv","value":info,"edi":edi.id})
				cr.commit()
				exception = True
		if not exception:
			cr.commit()
		for i in ids :
			cr2.execute("update edi_edi set thread = False where id = %s" % i)
		if thread :
			cr.close()
		cr2.close()
		return True
			
	def import_csv_thread(self, cr, uid, ids, context=False):
		cr2 = pooler.get_db(cr.dbname).cursor()
		thread_ptr = threading.Thread(target=self.import_csv, args=(cr2, uid, ids, context))
		thread_ptr.start()
		return {}
		
	def import_csv(self, cr, uid, ids, context=False):
		info = ""
		thread = False
		if "thread" in context:
			thread = context["thread"]
		cr2 = pooler.get_db(cr.dbname).cursor()
		cr2.autocommit(True)
		exception = False
		if not ids :
			if 'edi.code' in context:
				ids = self.search(cr,uid,[('code','=',context['edi.code'])])
			else:
				return False
		for edi in self.browse(cr,uid,ids,context):
			self.write(cr2,uid,[edi.id],{'thread':True})
			try :
				structs = edi._create_struct_csv()[edi.id]
				if not edi.eval_in:
					continue
				edi.eval_in = edi.eval_in.replace(chr(13),"\n")
				data = {}
				gobreak = False
				for filename in edi._get_files(edi.path_in, edi.file_template):
					result = ""
					_logger.info("Import file : %s" % (filename))
					context['edi'] = edi.id
					context['type'] = 'import'
					context['filename'] = filename
					try:
						file = codecs.open(filename,'rb',edi.charset)
						file_csv = unicode_csv_reader(file, delimiter=str(edi.delimiter[0]), quotechar=str(edi.quotechar))
						data[filename] = []
						nb_line = 0
						first = True
						for line in file_csv:
							nb_line+=1
							_logger.info("Import line : %s" % (str(nb_line)))
							if first and edi.skip_first:
								nb_line = 0
								first = False
								continue
							if edi.line_start > 0 and nb_line < edi.line_start:
								continue
							if edi.line_stop > 0 and nb_line > edi.line_stop:
								break
							data[filename].append(edi._parse_line_csv(line,structs))
							vals_import = edi._parse_line_csv(line,structs)
							exec edi.eval_in
							if gobreak :
								break
						file.close()
					except Exception, e:
						if not os.path.exists (edi.path_in+"error/"):
							os.makedirs(edi.path_in+"error/")
						shutil.move(edi.path_in+os.path.basename(filename), edi.path_in+"error/"+os.path.basename(filename))
						raise e
					if edi.path_in_move:
						shutil.move(edi.path_in+os.path.basename(filename), edi.path_in_move+os.path.basename(filename))	
					_logger.info("End Import file : %s" % (filename))
					if result :
						self.pool.get('edi.edi.result').create(cr,uid,{"name":"_import_csv","value":result,"edi":edi.id})
			except Exception, e:
				import sys,traceback
				tb = sys.exc_info()
				tb_s = "".join(traceback.format_exception(*tb))
				info = tb_s
				try:
					info += "\n"+ustr(e.value)
				except:
					pass
#				info = "\n"+ustr(e.exc_type)+"\n"+ustr(e.value)+"\n"+ustr(e) +"\n" +reduce(lambda x, y: x+y, traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback))
				_logger.info("Except : %s" % (info))
				cr.rollback()
				self.pool.get('edi.edi.result').create(cr,uid,{"name":"_import_csv","value":info,"edi":edi.id})
				cr.commit()
				exception = True
		if not exception:
			cr.commit()
		for i in ids :
			cr2.execute("update edi_edi set thread = False where id = %s" % i)
		if thread :
			cr.close()
		cr2.close()
		if not thread and exception:
			raise Exception(info)
		return True
	
	def export_csv_thread(self, cr, uid, ids, context=False):
		cr2 = pooler.get_db(cr.dbname).cursor()
		thread_ptr = threading.Thread(target=self.export_csv, args=(cr2, uid, ids, context))
		thread_ptr.start()
		return {}
		
	def export_csv(self, cr, uid, ids, context=False):
		thread = False
		if "thread" in context:
			thread = context["thread"]
		cr2 = pooler.get_db(cr.dbname).cursor()
		cr2.autocommit(True)
		exception = False
		if not ids :
			if 'edi.code' in context:
				ids = self.search(cr,uid,[('code','=',context['edi.code'])])
			else:
				return False
			
		for edi in self.browse(cr,uid,ids,context):
			self.write(cr2,uid,[edi.id],{'thread':True})
			cr2.commit()
			try:
				structs = edi._create_struct_csv()[edi.id]
				if not edi.eval_out:
					continue
				edi.eval_out = edi.eval_out.replace(chr(13),"\n")
				data = {}
				context['edi'] = edi.id
				context['type'] = 'export'
				exec edi.eval_out 
				context['path_out'] = edi.path_out
				edi._create_files_csv(data=data,context=context)
			except:
				import sys,traceback
				tb = sys.exc_info()
				tb_s = "".join(traceback.format_exception(*tb))
				info = tb_s
#				info = reduce(lambda x, y: x+y, traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback))
				_logger.info("Except : %s" % (info))
				cr.rollback()
				self.pool.get('edi.edi.result').create(cr,uid,{"name":"_import_csv","value":info,"edi":edi.id})
				cr.commit()
				exception = True
		if not exception:
			cr.commit()
		for i in ids :
			cr2.execute("update edi_edi set thread = False where id = %s" % i)
		if thread :
			cr.close()
		cr2.close()
		return True
	
#	def import_xml_thread(self, cr, uid, ids, context=False):
#		cr2 = pooler.get_db(cr.dbname).cursor()
#		thread_ptr = threading.Thread(target=self.import_xml, args=(cr2, uid, ids, context))
#		thread_ptr.start()
#		return {}
#		
#	def import_xml(self, cr, uid, ids, context=False):
#		info = ""
#		thread = False
#		if "thread" in context:
#			thread = context["thread"]
#		cr2 = pooler.get_db(cr.dbname).cursor()
#		cr2.autocommit(True)
#		exception = False
#		if not ids :
#			if 'edi.code' in context:
#				ids = self.search(cr,uid,[('code','=',context['edi.code'])])
#			else:
#				return False
#		for edi in self.browse(cr,uid,ids,context):
#			self.write(cr2,uid,[edi.id],{'thread':True})
#			try :
#				structs = edi._create_struct_xml()[edi.id]
#				if not edi.eval_in:
#					continue
#				edi.eval_in = edi.eval_in.replace(chr(13),"\n")
#				data = {}
#				gobreak = False
#				for filename in edi._get_files(edi.path_in, edi.file_template):
#					result = ""
#					_logger.info("Import file : %s" % (filename))
#					context['edi'] = edi.id
#					context['type'] = 'import'
#					context['filename'] = filename
#					try:
#						file = codecs.open(filename,'rb',edi.charset)
#						file_xml = unicode_xml_reader(file, delimiter=str(edi.delimiter[0]), quotechar=str(edi.quotechar))
#						data[filename] = []
#						nb_line = 0
#						first = True
#						for line in file_xml:
#							nb_line+=1
#							_logger.info("Import line : %s" % (str(nb_line)))
#							if first and edi.skip_first:
#								nb_line = 0
#								first = False
#								continue
#							if edi.line_start > 0 and nb_line < edi.line_start:
#								continue
#							if edi.line_stop > 0 and nb_line > edi.line_stop:
#								break
#							data[filename].append(edi._parse_line_xml(line,structs))
#							vals_import = edi._parse_line_xml(line,structs)
#							exec edi.eval_in
#							if gobreak :
#								break
#						file.close()
#					except Exception, e:
#						if not os.path.exists (edi.path_in+"error/"):
#							os.makedirs(edi.path_in+"error/")
#						shutil.move(edi.path_in+os.path.basename(filename), edi.path_in+"error/"+os.path.basename(filename))
#						raise e
#					if edi.path_in_move:
#						shutil.move(edi.path_in+os.path.basename(filename), edi.path_in_move+os.path.basename(filename))	
#					_logger.info("End Import file : %s" % (filename))
#					if result :
#						self.pool.get('edi.edi.result').create(cr,uid,{"name":"_import_xml","value":result,"edi":edi.id})
#			except Exception, e:
#				import sys,traceback
#				tb = sys.exc_info()
#				tb_s = "".join(traceback.format_exception(*tb))
#				info = tb_s
#				try:
#					info += "\n"+ustr(e.value)
#				except:
#					pass
##				info = "\n"+ustr(e.exc_type)+"\n"+ustr(e.value)+"\n"+ustr(e) +"\n" +reduce(lambda x, y: x+y, traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback))
#				_logger.info("Except : %s" % (info))
#				cr.rollback()
#				self.pool.get('edi.edi.result').create(cr,uid,{"name":"_import_xml","value":info,"edi":edi.id})
#				cr.commit()
#				exception = True
#		if not exception:
#			cr.commit()
#		for i in ids :
#			cr2.execute("update edi_edi set thread = False where id = %s" % i)
#		if thread :
#			cr.close()
#		cr2.close()
#		if not thread and exception:
#			raise Exception(info)
#		return True
	
	def _create_files_xml(self, cr, uid, ids, data, context=False):
		edi = self.browse(cr,uid,ids,context)[0]
		for filename in data: 
			if not data[filename]:
				continue
#			file = codecs.open(context['path_out']+filename,"w",edi.charset)
			file = open(context['path_out']+filename,"wb")
			file.write(dict2xml(data[filename]))
#			csv_writer = UnicodeWriter(file, encoding=edi.charset, delimiter=str(edi.delimiter[0]), quotechar=str(edi.quotechar), quoting=csv.QUOTE_NONNUMERIC)
#			for line in data[filename]:
#				csv_writer.writerow(line)
			file.close()
			edi._create_attachment(file_path=context['path_out']+filename,context=context)
			if edi.copy_out:
				print "TODO copy out"
		return {}
	
	def export_xml_thread(self, cr, uid, ids, context=False):
		cr2 = pooler.get_db(cr.dbname).cursor()
		thread_ptr = threading.Thread(target=self.export_xml, args=(cr2, uid, ids, context))
		thread_ptr.start()
		return {}
		
	def export_xml(self, cr, uid, ids, context=False):
		thread = False
		if "thread" in context:
			thread = context["thread"]
		cr2 = pooler.get_db(cr.dbname).cursor()
		cr2.autocommit(True)
		exception = False
		if not ids :
			if 'edi.code' in context:
				ids = self.search(cr,uid,[('code','=',context['edi.code'])])
			else:
				return False
			
		for edi in self.browse(cr,uid,ids,context):
			self.write(cr2,uid,[edi.id],{'thread':True})
			cr2.commit()
			try:
#				structs = edi._create_struct_xml()[edi.id]
				if not edi.eval_out:
					continue
				edi.eval_out = edi.eval_out.replace(chr(13),"\n")
				data = {}
				context['edi'] = edi.id
				context['type'] = 'export'
				exec edi.eval_out 
				context['path_out'] = edi.path_out
				edi._create_files_xml(data=data,context=context)
			except:
				import sys,traceback
				tb = sys.exc_info()
				tb_s = "".join(traceback.format_exception(*tb))
				info = tb_s
#				info = reduce(lambda x, y: x+y, traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback))
				_logger.info("Except : %s" % (info))
				cr.rollback()
				self.pool.get('edi.edi.result').create(cr,uid,{"name":"_import_xml","value":info,"edi":edi.id})
				cr.commit()
				exception = True
		if not exception:
			cr.commit()
		for i in ids :
			cr2.execute("update edi_edi set thread = False where id = %s" % i)
		if thread :
			cr.close()
		cr2.close()
		return True
	
			
	def _get_log(self, cr, uid, ids, context=False):
		edi = self.read(cr,uid,context['edi'],['force'])
		if not edi['force'] :
			cr.execute("select object_id from edi_edi_log where edi = %s" % str(context['edi']) )
			res = cr.dictfetchall()
			for log in res:
				(model_name, id) = log['object_id'].split(',')
				id = int(id)
				if model_name == context['model_log'] and id in context['ids_log']:
					context['ids_log'].remove(id)
		return context['ids_log']

	def _get_log_ext(self, cr, uid, ids, context=False):
		self.pool.get(context['model_log'])
		edi = self.browse(cr,uid,context['edi'])
		if not edi.force :
			for log in edi.log_ids:
				if log.ref == context['ref']:
					return False
		return True
		
	def _log(self, cr, uid, ids, context=False):
		log_obj = self.pool.get('edi.edi.log')
		vals = {
			'name':context['filename'],
			'edi':context['edi'],
			'object_id' :context['model_log']+','+str(context['id_log']),
			'type':context['type'],
			'ref':'ref' in context and context['ref'],
		}
		log_obj.create(cr,uid,vals)
edi_edi()

class edi_edi_log (osv.osv):
	_name = 'edi.edi.log'
	_description = 'edi.edi.log'
	
	def _models_get(self, cr, uid, context={}):
		obj = self.pool.get('ir.model')
		ids = obj.search(cr, uid, [])
		res = obj.read(cr, uid, ids, ['model', 'name'], context)
		return [(r['model'], r['name']) for r in res]
   
	_columns = {
		'name':fields.char('Name', size=128, required=True),
		'date': fields.datetime('Date'),
		'edi':fields.many2one('edi.edi', 'Edi'),
		'object_id' : fields.reference('Object', selection=_models_get, size=64),
		'type': fields.selection([('export','export'),('import','import')], 'Type', required=True, size=24),
		'ref':fields.char('Ref', size=128),
	}
	_defaults = {
		"date": lambda *a: time.strftime("%Y-%m-%d %H:%M:%S"),
	}
	_order = "date desc"
edi_edi_log()

class edi_edi_result (osv.osv):
	_name = 'edi.edi.result'
	_description = 'edi.edi.result'
	
	_columns = {
		'name':fields.char('Name', size=128, required=True),
		'date': fields.datetime('Date'),
		'edi':fields.many2one('edi.edi', 'Edi'),
		'value': fields.text('Value'),
	}
	_defaults = {
		"date": lambda *a: time.strftime("%Y-%m-%d %H:%M:%S"),
	}
	_order = "date desc"
edi_edi_result()

class edi_edi_line_edi (osv.osv):
	_name = 'edi.edi.line.edi'
	_description = 'edi.edi.line.edi'
	
	def check_name(self, cr, uid, ids):
		for line in self.browse(cr,uid,ids):
			if len(line.name) != int(line.edi.length_identifier) and int(line.edi.length_identifier) != 0:
				return False
		return True
	
	def _dict_fields_ids(self, cr, uid, ids, name, arg, context=None):
		res = {}
		for line in self.browse(cr, uid, ids, context=context):
			res[line.id] = ""
			for field in line.fields_ids:
				res[line.id] += "\""+field.name+"\":False,\r"
				
			char = '0'
			for field in line.fields_ids:
				res[line.id] += char*field.length
				if char == '0':
					char = '1'
				else:
					char = '0'
		return res
	
	_columns = {
		'name':fields.char('Name', size=128, required=True),
		'sequence': fields.integer('Sequence', required=True),
		'edi':fields.many2one('edi.edi', 'Line'),
		'fields_ids':fields.one2many('edi.edi.line.edi.field', 'line', 'Fields'),
		'dict': fields.function(_dict_fields_ids, readonly=False, method=True, store=False, string='Dict to put in eval', type='text'),
	}
	_defaults = {
		'sequence': lambda *a : 1,
	}
	_order = "sequence"
	
	_constraints = [(check_name, "Invalid length for name", ["name"])]
	
edi_edi_line_edi()

class edi_edi_line_edi_field (osv.osv):
	_name = 'edi.edi.line.edi.field'
	_description = 'edi.edi.line.edi.field'
	
	def check_start_stop(self, cr, uid, ids):
		for field in self.browse(cr,uid,ids):
			if field.start > field.stop:
				return False
			for next in field.line.fields_ids:
				if field.start >= next.start and field.start <= next.stop and field.id != next.id:
					return False
				elif field.stop >= next.start and field.stop <= next.stop and field.id != next.id:
					return False
		return True
	
	def check_name(self, cr, uid, ids):
		for field in self.browse(cr,uid,ids):
			for next in field.line.fields_ids:
				if field.name == next.name and field.id != next.id:
					return False
		return True
	
	def _length(self, cr, uid, ids, name, arg, context=None):
		res = {}
		for field in self.browse(cr, uid, ids, context=context):
			res[field.id] = field.stop - field.start + 1 
		return res
	
	_columns = {
		'name':fields.char('Name', size=128, required=True),
		'line':fields.many2one('edi.edi.line.edi', 'Line'),
		'start':fields.integer('Start', required=True),
		'stop':fields.integer('Stop', required=True),
		'length': fields.function(_length, readonly=True, method=True, store=False, string='Length', type='integer'),
		'align': fields.selection([('right','right'),('left','left'),('zero','zero')], 'Align', required=True, size=24),
		'type': fields.selection([('int','int'),('float','float'),('char','char')], 'Type  (not implemented)', required=True, size=24),
		'cast':fields.char('Cast  (not implemented)', size=128,),
		'description':fields.char('Description', size=128,),
	}
	_defaults = {
	}
	_order = "start"
	
	_constraints = [(check_start_stop, "Invalid start stop", ["start","stop"]),(check_name, "The name must be unique !", ["name"])]
edi_edi_line_edi_field()

class edi_edi_csv_field (osv.osv):
	_name = 'edi.edi.csv.field'
	_description = 'edi.edi.csv.field'
	
	def check_name(self, cr, uid, ids):
		for field in self.browse(cr,uid,ids):
			for next in field.edi.fields_ids:
				if field.name == next.name and field.id != next.id:
					return False
		return True
	
	def check_sequence(self, cr, uid, ids):
		for field in self.browse(cr,uid,ids):
			for next in field.edi.fields_ids:
				if field.sequence == next.sequence and field.id != next.id:
					return False
		return True
	
	_columns = {
		'name':fields.char('Name', size=128, required=True),
		'sequence': fields.integer('Sequence', required=True),
		'edi':fields.many2one('edi.edi', 'Edi'),
		'type': fields.selection([('int','int'),('float','float'),('char','char')], 'Type  (not implemented)', required=True, size=24),
		'cast':fields.char('Cast  (not implemented)', size=128,),
		'description':fields.char('Description', size=128,),
	}
	_defaults = {
		'sequence': lambda *a : 1
	}
	_order = "sequence"
	
	_constraints = [(check_name, "The name must be unique !", ["name"]),(check_sequence, "The sequence must be unique !", ["sequence"])]
edi_edi_csv_field()

class edi_edi_xml_field (osv.osv):
	_name = 'edi.edi.xml.field'
	_description = 'edi.edi.xml.field'
	
	def check_name(self, cr, uid, ids):
		for field in self.browse(cr,uid,ids):
			for next in field.edi.fields_ids:
				if field.name == next.name and field.id != next.id:
					return False
		return True
	
	_columns = {
		'name':fields.char('Name', size=128, required=True),
		'sequence': fields.integer('Sequence', required=True),
		'parent':fields.many2one('edi.edi.xml.field', 'Parent'),
		'edi':fields.many2one('edi.edi', 'Edi'),
		'type': fields.selection([('int','int'),('float','float'),('char','char')], 'Type  (not implemented)', required=True, size=24),
		'cast':fields.char('Cast  (not implemented)', size=128,),
		'description':fields.char('Description', size=128,),
	}
	_defaults = {
		'sequence': lambda *a : 1
	}
	_order = "parent,sequence"
	
	_constraints = [(check_name, "The name must be unique !", ["name"])]
edi_edi_xml_field()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: