# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2010-2012 ASPerience SARL (<http://www.asperience.fr>). 
#    All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
import os, base64
from tools import config
import decimal_precision as dp

class import_folder(osv.osv_memory):
    _name = "edi.folder"
    _description = "Folder"

    _columns = {
        'name': fields.char('Name', size=64),
        'folder_ids' : fields.one2many('edi.folder', 'folder_id', 'Folders'),
        'file_ids' : fields.one2many('edi.file', 'folder_id', 'Files'),
        'folder_id': fields.many2one('edi.folder', 'Folder'),
    }
    
    def search(self, cr, user, args, offset=0, limit=None, order=None, context=None, count=False):
#        print "search",args
        res = super(import_folder, self).search(cr, user, args=args, offset=0, limit=0, order=order,context=context, count=count)
        if not args or args == [('folder_id', '=', False)] or args == [['folder_id', '=', False]]:
            if not res:
                self.create(cr,user,{})
#            else:
#                self.unlink(cr,user,res)
#                self.create(cr,user,{})
            res = super(import_folder, self).search(cr, user, args=args, offset=0, limit=0, order=order,context=context, count=count)
        return res
    
    def create(self, cr, uid, vals, context=None):
#        print "create"
        result = super(import_folder, self).create(cr, uid, vals, context)
        obj = self.browse(cr,uid,result,context)
        def get_path(obj):
            result = obj.name
            if obj.folder_id:
                result = get_path(obj.folder_id)+"/"+result
            return result
        path = config.get('edi_path', '/tmp/')+cr.dbname.strip()+"/"
        if not os.path.exists(path):
            os.mkdir(path)
        path = path+get_path(obj)

        for fname in os.listdir(path):
            if fname[0] == '.':
                continue
            if os.path.isdir(path+"/"+fname):
                vals = {
                    "name": fname,
                    "folder_id": obj.id,
                }
                self.create(cr,uid,vals,context)
            else:
                vals = {
                    "name": fname,
                    "folder_id": obj.id,
                }
                self.pool.get("edi.file").create(cr,uid,vals,context)
        
        return result
    
    _defaults = {
        'name': ".",
        'folder_id': False,
    }
import_folder ()

class import_file(osv.osv_memory):
    _name = "edi.file"
    _description = "File"

    _columns = {
        'name': fields.char('Name', size=64),
        'file': fields.binary('File', required=True),
        'folder_id': fields.many2one('edi.folder', 'Folder'),
    }
    
    def create(self, cr, uid, vals, context=None):
        if 'file' in vals and 'folder_id' in vals:
            obj = self.pool.get("edi.folder").browse(cr,uid,vals['folder_id'],context)
            def get_path(obj):
                result = obj.name
                if obj.folder_id:
                    result = get_path(obj.folder_id)+"/"+result
                return result
            path = config.get('edi_path', '/tmp/')+cr.dbname.strip()+"/"
            path = path+get_path(obj)
            ff =base64.decodestring(vals['file'])
            fp = open(path+"/"+vals['name'],'wb')
            fp.write(ff)
            fp.close()
        result = super(import_file, self).create(cr, uid, vals, context)
        return result
        
    def write(self, cr, uid, ids, vals, context=None):
#        print "write"
        result = super(import_file,self).write(cr, uid, ids, vals, context)
        return result
    
    _defaults = {
    }

import_file()

class import_import(osv.osv_memory):
    _name = "edi.import"
    _description = "Import"

    _columns = {
        'name': fields.char('Name', size=256),
        'path': fields.char('path', size=256),
        'file': fields.binary('File', required=True),
    }
    
    def default_get(self, cr, uid, fields, context={}):
        res = super(import_import, self).default_get(cr, uid, fields, context=context)
        edi = self.pool.get("edi.edi").browse(cr,uid,context['active_id'])
        if edi.path_in:
            res['path']=edi.path_in
        return res
    
    def onchange_file(self, cr, uid, ids, name):
        name = name.replace('C:\\fakepath\\','')
        return {'value':{'name':name}}
        
    def ok(self, cr, uid, ids, context={}):
        for id in ids:
            ptr = self.browse(cr,uid,id)
            vals =base64.decodestring(ptr.file)
            if ptr.path[-1] != '/':
                ptr.path += '/'
            fp = open(ptr.path+ptr.name,'wb')
            fp.write(vals)
            fp.close()
        return {'type': 'ir.actions.act_window_close'}
    
    _defaults = {
    }

import_import()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: