# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2010-2012 ASPerience SARL (<http://www.asperience.fr>). 
#    All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
import os, base64
from tools import config
import tools
import decimal_precision as dp


class wizard_fusion(osv.osv_memory):
    _name = "wizard.fusion"
    _description = "wizard.fusion"

    _columns = {
        'name': fields.char('Name', size=64),
        'partner_ids': fields.many2many('res.partner', "wizard_fusion_partner_rel", "wiz_id", "part_id"),
        'partner_id': fields.many2one('res.partner','Partner Destination'),
        'product_ids': fields.many2many('product.product', "wizard_fusion_product_rel", "wiz_id", "prod_id"),
        'product_id': fields.many2one('product.product','Product Destination'),
        'diff': fields.text('diff'),
    }
    
    def _get_partner_ids(self, cr, uid, context=None):
        if context['active_model'] == 'res.partner':
            return context['active_ids']
        return False
    
    def _get_product_ids(self, cr, uid, context=None):
        if context['active_model'] == 'product.product':
            return context['active_ids']
        return False
    
    def _get_name(self, cr, uid, context=None):
        return context['active_model']
  
    def _get_diff(self, cr, uid, context=None):
        if len(context['active_ids']) >= 1:
            obj = self.pool.get(context['active_model'])
            read_objs = obj.read(cr,uid,context['active_ids'])
            diff = ""
            for key in sorted(read_objs[0].keys()):
                ok = True
                value=read_objs[0][key]
                for read_obj in read_objs[1:]:
                    if read_obj[key] != value:
                        ok = False
                if not ok:
                    diff += key.ljust(50)+" --> "+tools.ustr(value).ljust(50)
                    for read_obj in read_objs[1:]:
                        diff +="| "+tools.ustr(read_obj[key]).ljust(50)+"\n"
            return diff
        return False 
    
    _defaults = {
        'name': _get_name,
        'partner_ids': _get_partner_ids,
        'product_ids': _get_product_ids,
        'diff': _get_diff,
    }
    
    def fusion(self, cr, uid, ids, context=None):
        for wizard in self.browse(cr,uid,ids,context):
            log = ""
            obj_table = wizard.name.replace('.','_')
            obj_ids = False
            obj_id = False
            if wizard.name == "res.partner":
                obj_ids = wizard.partner_ids
                obj_id = wizard.partner_id
                
                sql = " select b.relname as table,c.relname,d.attname as column \
                        from pg_constraint a \
                             join pg_class b on a.conrelid=b.oid \
                             join pg_class c on a.confrelid=c.oid \
                             join pg_attribute d on d.attnum = any(a.conkey) and b.oid=d.attrelid \
                             where c.relname = '"+obj_table+"'"
                             
                cr.execute(sql)
                res = cr.dictfetchall()
                for obj in obj_ids:
                    if obj.id != obj_id.id:
                        for table in res:
                            if table['table'][-4:] == '_rel':
                                sql = "select b.relname as table,c.relname,d.attname as column \
                                from pg_constraint a \
                                     join pg_class b on a.conrelid=b.oid \
                                     join pg_class c on a.confrelid=c.oid \
                                     join pg_attribute d on d.attnum = any(a.conkey) and b.oid=d.attrelid \
                                where b.relname = '%s'" % table['table']
                                cr.execute(sql)
                                res2 = cr.dictfetchall()
                                if len(res2) == 2:
                                    field1 = res2[0]['column']
                                    field2 = res2[1]['column']  
                                    sql = """delete from %s where oid in (
                                                select b.oid
                                                from %s b
                                                where exists( select 1
                                                from %s a
                                                where a.oid != b.oid and
                                                a.%s = b.%s and
                                                a.%s = b.%s and
                                                a.oid < b.oid )
                                    )""" % (table['table'],table['table'],table['table'],field1,field1,field2,field2)
                                    cr.execute(sql)
                                    sql = """DELETE FROM %s WHERE %s = %s """%(table['table'], table['column'], obj.id)
                                    if cr.execute(sql) != None:
                                        log += "Erreur sur : "+str(sql)+","
                            else:
                                sql = """UPDATE %s SET %s = %s WHERE %s = %s """%(table['table'], table['column'], obj_id.id,table['column'], obj.id)
                                if cr.execute(sql) != None:
                                    log += "Erreur sur : "+str(sql)+","
                                     
                        obj.write({'active':False})
            elif wizard.name == "product.product":
                obj_ids = wizard.product_ids
                obj_id = wizard.product_id
                tmpl_id = wizard.product_id.product_tmpl_id
                
                sql = " select b.relname as table,c.relname,d.attname as column \
                        from pg_constraint a \
                             join pg_class b on a.conrelid=b.oid \
                             join pg_class c on a.confrelid=c.oid \
                             join pg_attribute d on d.attnum = any(a.conkey) and b.oid=d.attrelid \
                             where c.relname = '"+"product_template"+"'"
                             
                cr.execute(sql)
                res = cr.dictfetchall()
                for obj in obj_ids:
                    if obj.id != obj_id.id:
                        for table in res:
                            if table['table'] == 'product_product' and table['column'] == 'product_tmpl_id':
                                continue
                                
                            if table['table'][-4:] == '_rel':
                                sql = "select b.relname as table,c.relname,d.attname as column \
                                from pg_constraint a \
                                     join pg_class b on a.conrelid=b.oid \
                                     join pg_class c on a.confrelid=c.oid \
                                     join pg_attribute d on d.attnum = any(a.conkey) and b.oid=d.attrelid \
                                where b.relname = '%s'" % table['table']
                                cr.execute(sql)
                                res2 = cr.dictfetchall()
                                if len(res2) == 2:
                                    field1 = res2[0]['column']
                                    field2 = res2[1]['column']  
                                    sql = """delete from %s where oid in (
                                                select b.oid
                                                from %s b
                                                where exists( select 1
                                                from %s a
                                                where a.oid != b.oid and
                                                a.%s = b.%s and
                                                a.%s = b.%s and
                                                a.oid < b.oid )
                                    )""" % (table['table'],table['table'],table['table'],field1,field1,field2,field2)
                                    cr.execute(sql)
                                    sql = """DELETE FROM %s WHERE %s = %s """%(table['table'], table['column'], obj.id)
                                    if cr.execute(sql) != None:
                                        log += "Erreur sur : "+str(sql)+","
                            else:
                                sql = """UPDATE %s SET %s = %s WHERE %s = %s """%(table['table'], table['column'], tmpl_id.id,table['column'], obj.product_tmpl_id.id)
                                if cr.execute(sql) != None:
                                    log += "Erreur sur : "+str(sql)+","
                                        
                                
                sql = " select b.relname as table,c.relname,d.attname as column \
                        from pg_constraint a \
                             join pg_class b on a.conrelid=b.oid \
                             join pg_class c on a.confrelid=c.oid \
                             join pg_attribute d on d.attnum = any(a.conkey) and b.oid=d.attrelid \
                             where c.relname = '"+obj_table+"'"
                             
                cr.execute(sql)
                res = cr.dictfetchall()
                for obj in obj_ids:
                    if obj.id != obj_id.id:
                        for table in res:
                            if table['table'][-4:] == '_rel':
                                sql = "select b.relname as table,c.relname,d.attname as column \
                                from pg_constraint a \
                                     join pg_class b on a.conrelid=b.oid \
                                     join pg_class c on a.confrelid=c.oid \
                                     join pg_attribute d on d.attnum = any(a.conkey) and b.oid=d.attrelid \
                                where b.relname = '%s'" % table['table']
                                cr.execute(sql)
                                res2 = cr.dictfetchall()
                                if len(res2) == 2:
                                    field1 = res2[0]['column']
                                    field2 = res2[1]['column']  
                                    sql = """delete from %s where oid in (
                                                select b.oid
                                                from %s b
                                                where exists( select 1
                                                from %s a
                                                where a.oid != b.oid and
                                                a.%s = b.%s and
                                                a.%s = b.%s and
                                                a.oid < b.oid )
                                    )""" % (table['table'],table['table'],table['table'],field1,field1,field2,field2)
                                    cr.execute(sql)
                                sql = """DELETE FROM %s WHERE %s = %s """%(table['table'], table['column'], obj.id)
                                if cr.execute(sql) != None:
                                    log += "Erreur sur : "+str(sql)+","
                            else:
                                sql = """UPDATE %s SET %s = %s WHERE %s = %s """%(table['table'], table['column'], obj_id.id,table['column'], obj.id)
                                if cr.execute(sql) != None:
                                    log += "Erreur sur : "+str(sql)+","
                       
                        obj.write({'active':False})
            self.log(cr,uid,wizard.id,log)
                    
        return {'type': 'ir.actions.act_window_close'}
wizard_fusion ()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: